/*********************************************************************
 *
 * Authors: Vincenzo Ciaschini - Vincenzo.Ciaschini@cnaf.infn.it 
 *
 * Copyright (c) 2002, 2003 INFN-CNAF on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or
 * http://www.edg.org/license.html
 *
 * Parts of this code may be based upon or even include verbatim pieces,
 * originally written by other people, in which case the original header
 * follows.
 *
 *********************************************************************/
//#include "sqlwrap.h"
#include "mysqlwrap.h"
#include <string>

static ResUse dummy;

namespace sql {
  interface::interface(const char *dbname, const char *hostname, const char *user, const char *password) : con(true), err(0)
{
  try {
    con.connect(dbname, hostname, user, password);
  } catch (BadQuery) {
    throw DBEXC_Connect();
  } catch (...) {
    throw DBEXC();
  }
  err = 0;

}

interface::interface() : con(true), err(0) {}
interface::interface(interface const &iface) : con(iface.con), err(iface.err) {}

void interface::lock(std::string s)
{
  Query q(&con, true);
  q << s;
}

void interface::unlock(void)
{
  Query q(&con, true);
  q << "UNLOCK TABLES";
}

interface::~interface(void)
{
  //  for_each(activequerys.begin(), activequerys.end(), dlq);
}

int interface::error(void) const
{
  return err;
}

void interface::connect(const char *dbname, const char *hostname, const char *user, const char *password)
{
  try {
    con.connect(dbname, hostname, user, password);
  } catch (BadQuery) {
    throw DBEXC_Connect();
  } catch (...) {
    throw DBEXC();
  }
  err = 0;
}

#if 0
query &interface::newquery()
{
  query *q = new query(dynamic_cast<interface &>(this));
  //  activequerys.insert(q);
  return *q;
}
#endif

query::query(interface &face) : query_text(""), c(&face.con), q(&face.con, true)  {}
query::query(query const &query) : query_text(query.query_text), c(query.c), q(query.q) {}

query::~query(void)
{
  //  for_each(activeits.begin(), activeits.end(), dli);
}

query &query::operator<<(std::string s)
{
  std::string tmp = query_text + s;
  int    pos = tmp.find_last_of('\n');

  if (pos == -1)
    pos = 0;

  query_text = tmp.substr(pos, tmp.size() - pos);
  return *this;
}

void query::exec(void)
{
#if 0
  q << query_text;

  try {
    res = q.use();
  } catch (...) {
    throw DBEXC_Query();
  }
#endif
}

int query::error(void) const
{
  return 0;
}

#if 1
iterator query::begin(void)
{ 
  q << query_text;
  return iterator(&q);

#if 0
  iterator *it = new iterator(dynamic_cast<query const &>(this));
  //  activeits.insert(it);
  return *it;
#endif
}

iterator query::end(void)
{
  return iterator();
}
#endif

iterator::iterator(ResUse result): res(result)
{
  try {
    r = row(res.fetch_row());
  } catch (...) {
    throw DBEXC_Query();
  }
}
#if 1
iterator::iterator(Query *q)//: q(qry.c, true)//: res((qry.res)) 
{
  try {
    res = q->use();
    r = row(res.fetch_row());
  } catch (...) {
    throw DBEXC_Query();
  }
}
#endif

#if 0
iterator &query::end(void)
{
  iterator *it = new iterator();
  //  activeits.insert(it);
  return *it;
}
#endif

iterator::iterator(iterator const &it) : res(it.res), r(it.r) {}
iterator::iterator() : res(), r() {}
iterator::~iterator() {}

bool iterator::operator==(iterator &i) const
{
  return (this->r == i.r);
}

bool iterator::operator!=(iterator &i) const
{
  return !(this->r == i.r);
}

iterator &iterator::operator=(iterator &i)
{
  this->r = i.r;
  this->res = i.res;
  return *this;
}

iterator &iterator::operator++(void)
{
  try {
    Row realrow = res.fetch_row();

    if (realrow != Row())
      r = row(realrow);
    else
      r = row();
  } catch (...) {
    throw DBEXC_Query();
  }
  return *this;
}

iterator::pointer iterator::operator*(void) const
{
  return &r;
}

iterator::reference iterator::operator&(void) const
{
  return r;
}

row::row() : r() {}
row::row(row const &record) : r(record.r) {}
row::~row() {}
row::row(Row const &record) : r(record) {}

row &row::operator=(row const &record)
{
  r = record.r;
  return *this;
}

bool row::operator==(row const &r1) const
{
  return (this->r == r1.r);
}

bool row::operator!=(row const &r1) const
{
  return !(this->r == r1.r);
}


const std::string row::operator[](const int field) const
{
  return std::string(r[field]);
}

const std::string row::operator[](const std::string name) const
{
  return std::string(r[name]);
}

int row::size(void) const
{
  return r.size();
}

const std::string iterator::name(int index) const
{
  return res.names(index);
}
}

