// Request data transfer (and optionally wait till completed).

#include "std.h"
#include "datamove/datapoint.h"
#include "datamove/datahandle.h"
#include "misc/log_time.h"
#include "../userinterface/CertInfo.h"

// SSE
#include "https/se/file_soapH.h"
extern SOAP_NMAC struct Namespace file_soap_namespaces[];
#include "https/client/client.h"


bool sse_replicate(DataPoint& dest,list<string>& sources);

int main(int argc, char* argv[]) {
  LogTime::active=false;
  bool verbose = false;
  list<string> sources;
  for(;;) {
    opterr=0;
    int optc=getopt(argc,argv,"+hvd:s:");
    if(optc == -1) break;
    switch(optc) {
      case 'h': {
        olog<<"ngrequest [-h] [-v] [-d level] [-s source] destination"<<endl;
        olog<<" -h - print help"<<endl;
        olog<<" -v - print version"<<endl;
        olog<<" -d - set debug level"<<endl;
        olog<<" -s - source of data (can be multiple)"<<endl;
        return 1;
      }; break;
      case 'v': {
        olog<<"ngls: version "<<VERSION<<endl;
        return 0;
      };
      case 's': {
        sources.push_back(string(optarg));
      }; break;
      case 'd': {
        char* p;
        int i = strtol(optarg,&p,10);
        if(((*p) != 0) || (i<0)) {
          olog<<"Improper debug level '"<<optarg<<"'"<<endl;
          return 1;
        };
        verbose=true;
        LogTime::level=i;
      }; break;
      case '?': {
        olog<<"Unsupported option '"<<(char)optopt<<"'"<<endl;
        return 1;
      }; break;
      case ':': {
        olog<<"Missing parameter for option '"<<(char)optopt<<"'"<<endl;
        return 1;
      }; break;
      default: {
        olog<<"Undefined processing error"<<endl;
        return 1;
      };
    };
  };
  char* service_url = argv[optind+0];
  if(service_url == NULL) {
    olog<<"Missing service url" << endl; return 1;
  };
  if(argv[optind+1] != NULL) {
    olog<<"Unrecognized option: "<<argv[optind+1]<<endl; return 1;
  };
  DataPoint url(service_url);
  if(!url) {
    olog<<"Unsupported service url given"<<endl; return 1;
  };
  if(url.meta()) {
    olog<<"Meta-url for service is not supported yet"<<endl; return 1;
  };
  if(strncasecmp(service_url,"se://",5) == 0) {
    // send request to SSE 
    if(!sse_replicate(url,sources)) {
      olog<<"Failed"<<endl; return 1;
    };
  } else if(strncasecmp(service_url,"gsiftp://",9) == 0) {
    olog<<"GridFTP is not supported yet"<<endl; return 1;
  } else {
    olog<<"URL "<<service_url<<" is not supported"<<endl; return 1;
  };
  return 0;
}

bool sse_replicate(DataPoint& dest,list<string>& sources) {
  string service_url = dest.current_location();
  service_url.replace(0,2,"httpg");
  string service_path = "";
  {
    int n = service_url.find('?');
    if(n == string::npos) {
      odlog(0)<<"Missing LFN in destination URL"<<endl;
      return false;
    };
    service_path=service_url.c_str()+n+1;
    service_url.resize(n);
  };
  canonic_url(service_url); // remove options
  odlog(1)<<"Talking to SOAP service at "<<service_url<<endl;
  struct soap soap;
  HTTP_ClientSOAP s(service_url.c_str(),&soap);
  soap.namespaces=file_soap_namespaces;
  if(s.connect() != 0) {
    odlog(0)<<"Failed to connect to "<<service_url<<endl;
    return false;
  };
  ns__fileinfo info;
  ns__addResponse rr;
  int soap_err = SOAP_OK;
  info.id=(char*)(service_path.c_str());
  string acl("");
  {
    CertInfo ci;
    acl=ci.GetSN();
    if(acl.length()) acl="<gacl><entry><person><dn>"+acl+"</dn></person><allow><read/><write/><list/><admin/></allow></entry></gacl>";
    info.acl=(char*)(acl.c_str());
  };
  // unsigned long long int* size;
  // char* checksum;
  // char* created;
  // enum ns__filestate* state;
  // int __size_url;
  // char** url;
  int size_sources_ = sources.size();
  char** sources_;
  if(size_sources_ == 0) {
    sources_=(char**)malloc(sizeof(char*));
    sources_[0]=""; // replication
    size_sources_=1;
  } else {
    sources_=(char**)malloc(sizeof(char*)*size_sources_);
    int i = 0;
    list<string>::iterator ii = sources.begin();
    for(;i<size_sources_;++i,++ii) sources_[i]=(char*)(ii->c_str());
  };
  if((soap_err=soap_call_ns__add(&soap,s.SOAP_URL(),"add",&info,
                           size_sources_,sources_,rr)) != SOAP_OK) {
    odlog(1)<<"Failed to execute remote soap call 'add' at "<<service_url<<endl;
    return false;
  };
  if(rr.error_code != 0) {
    odlog(1)<<"Failed ("<<rr.error_code<<") to create remote file "<<info.id<<" at "<<service_url<<endl;
    return false;
  };
  return true;
}

