#include <iostream>
#include <vector>
#include <sstream>

#include "JobSubWindow.h"

#if QT_MAJOR > 2
#include "xRSLHighlighter.h"
#include "qtextedit.h"
#else
#include "qmultilineedit.h"
#endif

#include "qfont.h"
#include "qpushbutton.h"
#include "qlayout.h"
#include "qlistbox.h"
#include "qlabel.h"

#include "ngui.h"

using namespace std;

JobSubWindow::JobSubWindow( QWidget* parent, const char* name ) : QWidget( parent, name )
{ 
  textFont = new QFont( "System", 11, QFont::Normal );

#if QT_MAJOR > 2
  textEditor = new QTextEdit( this );
  textEditor->setWrapPolicy( QTextEdit::Anywhere ); 
  textEditor->setTextFormat( Qt::PlainText );
  xRSLHighlighter* xRSLHighlight = new xRSLHighlighter( textEditor );
#else
  textEditor = new QMultiLineEdit( this );
  textEditor->setWrapPolicy( QMultiLineEdit::Anywhere ); 
#endif
  textEditor->setFont( *textFont );

  submitButton = new QPushButton( "Submit xRSL", this );
  connect( submitButton, SIGNAL( pressed( ) ), this, SLOT( submit( ) ) );

  updateClusters = new QPushButton( "Update cluster-list", this );
  connect( updateClusters, SIGNAL( pressed( ) ), this, SLOT( update( ) ) );

  clusterView = new QListBox( this );
  clusterView->setSelectionMode( QListBox::Multi );
  clusterView->insertItem( "hypatia.uio.no" );
  clusterView->insertItem( "grid.nbi.dk" );
  clusterView->insertItem( "farm.hep.lu.se" );
  clusterView->insertItem( "atlantis.nbi.dk" );
  connect( clusterView, SIGNAL( doubleClicked( QListBoxItem* ) ), this, SLOT( showInfo( QListBoxItem* ) ) ); 

  clusterSelect = new QPushButton( "Invert chosen clusters", this );
  connect( clusterSelect, SIGNAL( pressed( ) ), clusterView, SLOT( invertSelection( ) ) );

  xRSLLabel = new QLabel( this );
  xRSLLabel->setText( "xRSL window" );

  clusterLabel = new QLabel( this );
  clusterLabel->setText( "Cluster list" );

  screenLayout = new QGridLayout( this, 3, 3, 5 );
  screenLayout->setColStretch( 0, 4 );
  screenLayout->setColStretch( 1, 4 );
  screenLayout->setColStretch( 2, 2 );

  screenLayout->addMultiCellWidget( textEditor, 1, 1, 0, 1 );
  screenLayout->addWidget( xRSLLabel, 0, 0 );
  screenLayout->addWidget( clusterLabel, 0, 2 );
  screenLayout->addWidget( submitButton, 2, 0 );
  screenLayout->addWidget( updateClusters, 2, 1 );
  screenLayout->addWidget( clusterView, 1, 2 );
  screenLayout->addWidget( clusterSelect, 2, 2 );
};

JobSubWindow::~JobSubWindow()
{ 
  delete textEditor;
  delete textFont;
  delete screenLayout;
  delete clusterSelect;
  delete submitButton;
  delete clusterView;
  delete updateClusters;
  delete xRSLLabel;
  delete clusterLabel;
};

void JobSubWindow::setTextArea( QString text )
{
  textEditor->clear();
  textEditor->setText( text );
}

QString JobSubWindow::returnXrsl( )
{
  return textEditor->text();
}

void JobSubWindow::submit( )
{
  string xrsl( (const char*)textEditor->text() );
  if( xrsl.length()==0 ) {
    emit( linemessage( "No xRSL in window..." ) );
    return;
  } else {
    emit( linemessage( "Submitting xrsl..." ) );
  }

  vector<string> chosenClusters;
  for( int i=0; i<clusterView->numRows(); i++ ) {
    if( clusterView->isSelected( i ) == true ) {
      chosenClusters.push_back( (const char*)clusterView->text( i ) );
    }
  }

  vector<string> dummy;
  string dummystring;
  vector<string> xrsls;
  xrsls.push_back( xrsl );

  // submit job dropping output into stream
  stringstream stream;
  streambuf* sbo = cout.rdbuf(stream.rdbuf());
  streambuf* sbe = cerr.rdbuf(stream.rdbuf());
  int submitstatus = ngsubxx( dummy, xrsls, chosenClusters, dummy, dummy, dummystring, false, false, UNDEFINED, 2, true );
  cerr.rdbuf(sbe);
  cout.rdbuf(sbo);

  if( submitstatus==0 ) {
    emit( linemessage( "Job successfully submitted..." ) );
  } else {
    emit( linemessage( "Job not submitted..." ) );
  }
}

void JobSubWindow::showInfo( QListBoxItem* item )
{
  vector<string> cluster;
  cluster.push_back( (const char*)item->text() );

  vector<string> dummy;
  bool dummybool = false;

  stringstream stream;
  streambuf* sbo = cout.rdbuf(stream.rdbuf());
  streambuf* sbe = cerr.rdbuf(stream.rdbuf());
  int submitstatus = ngstatxx( dummy, dummy, cluster, dummy, dummy, false, dummy, true, true, UNDEFINED, 0, true );
  cerr.rdbuf(sbe);
  cout.rdbuf(sbo);
  
  cout << stream.str() << endl;
}

void JobSubWindow::update( )
{
}
